package com.codigoparallevar.minicards;

import android.content.Context;
import android.content.Intent;

import com.codigoparallevar.minicards.types.functional.Consumer;
import com.codigoparallevar.minicards.types.functional.Producer;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.codigoparallevar.minicards.ui_helpers.GetAsync;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.programaker.api.ProgramakerApi;
import com.programaker.api.data.ProgramakerCustomBlock;
import com.programaker.api.data.api_results.ProgramakerGetCustomBlocksResult;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import java.io.IOException;
import java.util.List;

public class CardActivity extends AppCompatActivity {

    public final static String INTENT = "com.codigoparallevar.minicards.CARD";
    public static final String CARD_PATH_KEY = "CARD_PATH";
    public static final String VISUALIZATION_MODE_KEY = "VISUALIZATION_MODE";
    public static final String DEVELOPER_VISUALIZATION_MODE = "DEVELOPER_VISUALIZATION_MODE";

    CanvasView canvasView;
    com.getbase.floatingactionbutton.AddFloatingActionButton AddPartButton;
    com.getbase.floatingactionbutton.FloatingActionButton SetDevModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton SetUserModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton CenterViewButton;

    com.getbase.floatingactionbutton.FloatingActionsMenu devFabMenu;
    com.getbase.floatingactionbutton.FloatingActionsMenu userFabMenu;
    com.getbase.floatingactionbutton.FloatingActionButton ShowDeckFromDevModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton ShowDeckFromUserModeButton;

    boolean devMode = false;
    FloatingActionButton removePartFab;
    private PartsHolder partsHolder;
    private ProgramakerApi ProgramakerApi;
    private ConfigManager Config;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        this.Config = new ConfigManager(this);
        this.ProgramakerApi = new ProgramakerApi();
        String token = this.Config.getToken();
        if (token != null) {
            this.ProgramakerApi.setToken(token);
        }

        new GetAsync<ProgramakerGetCustomBlocksResult>().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR,
                new Tuple2<>(new Producer<ProgramakerGetCustomBlocksResult>() {
                    @Override
                    public ProgramakerGetCustomBlocksResult get() {
                        return CardActivity.this.ProgramakerApi.fetchCustomBlocks();
                    }
                }, new Consumer<ProgramakerGetCustomBlocksResult>() {
                    @Override
                    public void apply(ProgramakerGetCustomBlocksResult result) {
                        if (result == null) {
                            Log.e("CARDActivity", "error retrieving custom blocks");
                        }
                        else {
                            Log.d("CARDActivity", "custom blocks: " + result.toString());
                        }
                    }
                }));


        // Hide action bar
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.hide();
        }

        setContentView(R.layout.activity_main);

        // Use manually controlled canvas
        canvasView = (CanvasView) findViewById(R.id.canvasView);
        canvasView.setParentActivity(this);

        // Initialize auxiliary elements
        partsHolder = new PartsHolder(this);

        removePartFab = (FloatingActionButton) findViewById(R.id.remove_part_fab);
        canvasView.setDropZone(
                removePartFab.getX(), removePartFab.getX() + removePartFab.getWidth(),
                removePartFab.getY(), removePartFab.getY() + removePartFab.getHeight());

        // Set callbacks to fab buttons
        devFabMenu = (com.getbase.floatingactionbutton.FloatingActionsMenu)
                findViewById(R.id.dev_mode_fab);
        userFabMenu = (com.getbase.floatingactionbutton.FloatingActionsMenu)
                findViewById(R.id.user_mode_fab);

        AddPartButton = (com.getbase.floatingactionbutton.AddFloatingActionButton)
                findViewById(R.id.add_part_button);
        AddPartButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                partsHolder.openAddPartModal(canvasView);
            }
        });

        CenterViewButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.center_view_button);
        CenterViewButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                canvasView.CenterView();
            }
        });

        SetDevModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.set_developer_mode_button);
        SetDevModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CardActivity.this.setDevMode(true);
            }
        });

        ShowDeckFromDevModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.show_deck_from_dev_mode_button);
        ShowDeckFromDevModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(DeckPreviewActivity.INTENT);
                CardActivity.this.startActivity(i);
            }
        });

        ShowDeckFromUserModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.show_deck_from_user_mode_button);
        ShowDeckFromUserModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(DeckPreviewActivity.INTENT);
                CardActivity.this.startActivity(i);
            }
        });

        SetUserModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.set_user_mode_button);
        SetUserModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CardActivity.this.setDevMode(false);
            }
        });

        String visualization_mode = this.getIntent().getStringExtra(VISUALIZATION_MODE_KEY);
        if ((visualization_mode != null) && (visualization_mode.equals(DEVELOPER_VISUALIZATION_MODE))) {
            devMode = true;
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        loadCard();
        canvasView.resume();
    }

    @Override
    protected void onPause() {
        super.onPause();

        canvasView.pause();
    }

    private void loadCard() {
        String cardPath = this.getIntent().getStringExtra(CARD_PATH_KEY);
        if (cardPath == null) {
            finish();
        }

        try {
            canvasView.loadCard(cardPath);
        } catch (IOException e) {
            e.printStackTrace();
            finish();
        } catch (ErrorLoadingCardException e) {
            e.printStackTrace();
            finish();
        }
        setDevMode(devMode);
    }


    private void setDevMode(boolean goToDevMode) {
        boolean willInvalidate = devMode != goToDevMode;

        devMode = goToDevMode;
        if (goToDevMode){
            devFabMenu.setVisibility(View.VISIBLE);
            userFabMenu.setVisibility(View.GONE);
        }
        else {
            devFabMenu.setVisibility(View.GONE);
            userFabMenu.setVisibility(View.VISIBLE);
        }

        this.canvasView.setDevMode(devMode);

        if (willInvalidate){
            invalidate();
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (canvasView != null){
            return canvasView.onTouchEvent(event);
        }

        Log.e("CardActivity", "CanvasView is null");

        return super.onTouchEvent(event);
    }

    public void invalidate() {
        if (canvasView != null){
            if (canvasView.isDragging()){
                devFabMenu.setVisibility(View.GONE);
                userFabMenu.setVisibility(View.GONE);
                removePartFab.setVisibility(View.VISIBLE);
                Log.d("Main", "Changing visibility!");
            }
            else {
                this.setDevMode(devMode);
                removePartFab.setVisibility(View.GONE);
                Log.d("Main", "Now changing visibility!");
            }
            canvasView.setDropZone(
                    removePartFab.getX(), removePartFab.getX() + removePartFab.getWidth(),
                    removePartFab.getY(), removePartFab.getY() + removePartFab.getHeight());
        }
    }

    public static void openCard(Context context, CardFile cardfile, String visualizationMode) {
        Intent i = new Intent(INTENT);
        i.putExtra(CARD_PATH_KEY, cardfile.getPath());
        if (visualizationMode != null) {
            i.putExtra(VISUALIZATION_MODE_KEY, visualizationMode);
        }
        context.startActivity(i);
    }
}
